from ...types import JSON as _JSON

class JSON(_JSON): ...

class _FormatTypeMixin:
    def bind_processor(self, dialect): ...
    def literal_processor(self, dialect): ...

class JSONIndexType(_FormatTypeMixin, _JSON.JSONIndexType): ...
class JSONPathType(_FormatTypeMixin, _JSON.JSONPathType): ...
